<?php

/**
 * @file
 * Verify CyberSource reply POST and update relevant transactions.
 */

/**
 * Callback function for posted CyberSource SAWM payment response.
 */
function commerce_cybersource_sawm_reply() {
  if (isset($_POST['req_reference_number']) && is_numeric($_POST['req_reference_number'])) {
    $order = commerce_order_load(check_plain($_POST['req_reference_number']));
  }
  else {
    watchdog('commerce_cybersource_sawm', 'Invalid reference number.');
    return;
  }

  if (isset($_POST['req_transaction_uuid']) && is_numeric($_POST['req_transaction_uuid'])) {
    $transaction = commerce_payment_transaction_load(check_plain($_POST['req_transaction_uuid']));
  }
  else {
    watchdog('commerce_cybersource_sawm', 'Invalid transaction UUID.');
    return;
  }

  if (commerce_cybersource_sawm_reply_validate($order, $transaction) !== FALSE) {
    // Perform any submit functions if necessary.
    commerce_cybersource_sawm_update_transaction($order, $transaction);
  }
  else {
    // Otherwise display the failure message and send the customer back to
    // the order payment page.
    $order = commerce_order_status_update($order, 'checkout_review');
    drupal_set_message(t('Payment failed at the payment server. Please review your information and try again. If issues persist please contact your issuing bank.'), 'error');
  }
  drupal_goto(commerce_checkout_order_uri($order));
}

/**
 * Validate user's arrival at the success page.
 *
 * If necessary, move the user back to the previous page.
 *
 * @return bool
 *   Returns TRUE if the signature for signed fields is valid, otherwise FALSE.
 */
function commerce_cybersource_sawm_reply_validate($order, $transaction) {
  // Validate the returned data against its signature.
  if (!commerce_cybersource_sawm_verify_callback($transaction)) {
    watchdog(
      'commerce_cybersource_sawm',
      'Order @order_number with remote id: @remote_id has failed signature validation.',
      array(
        '@order_number' => $order->order_id,
        '@remote_id' => check_plain($_POST['transaction_id']),
      )
    );
    return FALSE;
  }
  return TRUE;
}

/**
 * Validate data returned from CyberSource is not forged.
 *
 * @param array $data
 *   An array of data elements from the CyberSource POST reply.
 * @param array $payment_method
 *   The payment method information from the method_info() hook and settings.
 *
 * @return bool
 *   Returns TRUE if the signature hash sent from CyberSource is equal to the
 *   hmac of the signed_field_names with the secret key from the Secure
 *   Acceptance profile, otherwise FALSE.
 */
function commerce_cybersource_sawm_verify_callback($transaction) {
  $payment_method = commerce_payment_method_instance_load($transaction->instance_id);

  // Apparently, transactions can get this far with no secret_key set.
  if (isset($payment_method['settings']['secret_key'])) {
    if (isset($_POST['signed_field_names'])) {
      if (isset($_POST['signature'])) {
        $fields = explode(',', $_POST['signed_field_names']);

        foreach ($fields as $field) {
          $data_to_sign[] = $field . "=" . $_POST[$field];
        }
        $signed_string = _commerce_cybersource_sawm_hash($data_to_sign, $payment_method['settings']['secret_key']);
        return ($signed_string == $_POST['signature']);
      }
      else {
        watchdog('commerce_cybersource_sawm', 'Reply POST had no signature.');
      }
    }
    else {
      watchdog('commerce_cybersource_sawm', 'Reply POST had no signed field names.');
    }
  }
  else {
    watchdog('commerce_cybersource_sawm', 'The secret key is not setup.');
  }
  return FALSE;
}

/**
 * Save the transaction information returned from CyberSource.
 */
function commerce_cybersource_sawm_update_transaction($order, $transaction) {
  // Coder module wants $_POST values to be filtered.
  $decision = check_plain($_POST['decision']);
  $message = check_plain($_POST['message']);
  $currency = check_plain($_POST['req_currency']);
  $transaction_uuid = check_plain($_POST['req_transaction_uuid']);

  $transaction->payload[REQUEST_TIME] = $_POST;
  $transaction->remote_status = $decision;

  switch (strtoupper($decision)) {
    case 'ACCEPT':
      $transaction->remote_id = check_plain($_POST['transaction_id']);
      $transaction->amount = commerce_currency_decimal_to_amount(check_plain($_POST['auth_amount']), $currency);
      $transaction->status = COMMERCE_PAYMENT_STATUS_SUCCESS;
      $transaction->message = empty($message) ? t('The payment has completed.') : $message;

      // Send the customer on to the next checkout page.
      commerce_order_status_update($order, 'checkout_complete');
      commerce_checkout_complete($order);
      break;

    case 'CANCEL':
    case 'DECLINE':
    case 'REVIEW':
    case 'ERROR':
      $transaction->remote_id = check_plain($_POST['req_transaction_uuid']);
      $transaction->amount = commerce_currency_decimal_to_amount($_POST['req_amount'], $currency);
      $transaction->status = COMMERCE_PAYMENT_STATUS_FAILURE;
      $transaction->message = empty($message) ? t('The payment has failed.') : $message;

      $order = commerce_order_status_update($order, 'checkout_review');
      watchdog(
        'commerce_cybersource_sawm',
        'Order @order_number with remote id: @remote_id has failed with the following code @code:@code_message',
        array(
          '@order_number' => $order->order_id,
          '@remote_id' => $transaction_uuid,
          '@code' => $decision,
          '@code_message' => $message . (empty($_POST['invalid_fields']) ? '' : ' - ' . check_plain($_POST['invalid_fields'])),
        )
      );
      break;
  }

  /*
  @todo
  Save the payment_token for later use somewhere.
  */

  // Save the transaction information.
  commerce_payment_transaction_save($transaction);
  watchdog(
    'commerce_cybersource_sawm',
    'Order @order_number with remote id: @remote_id has been processed with the following code @code:@code_message',
    array(
      '@order_number' => $order->order_id,
      '@remote_id' => isset($_POST['transaction_id']) ? check_plain($_POST['transaction_id']) : $transaction_uuid,
      '@code' => $decision,
      '@code_message' => $message . (empty($_POST['invalid_fields']) ? '' : ' - ' . check_plain($_POST['invalid_fields'])),
    )
  );
}
